@@ -251,7 +251,7 @@ class Abstract_Wallet(object):
                 tx = self.transactions.get(tx_hash)
                 if tx is not None:
                     tx.deserialize()
-                    self.add_transaction(tx_hash, tx, tx_height)
+                    self.add_transaction(tx_hash, tx)
         if save:
             self.storage.put('addr_history', self.history, True)
 
@@ -424,7 +424,9 @@ class Abstract_Wallet(object):
         with self.lock:
             self.verified_tx[tx_hash] = info  # (tx_height, timestamp, pos)
         self.storage.put('verified_tx3', self.verified_tx, True)
-        self.network.trigger_callback('updated')
+
+        conf, timestamp = self.get_confirmations(tx_hash)
+        self.network.trigger_callback('verified', (tx_hash, conf, timestamp))
 
     def get_unverified_txs(self):
         '''Returns a list of tuples (tx_hash, height) that are unverified and not beyond local height'''
@@ -693,7 +695,7 @@ class Abstract_Wallet(object):
                     print_error("found pay-to-pubkey address:", addr)
                     return addr
 
-    def add_transaction(self, tx_hash, tx, tx_height):
+    def add_transaction(self, tx_hash, tx):
         is_coinbase = tx.inputs[0].get('is_coinbase') == True
         with self.transaction_lock:
             # add inputs
@@ -744,7 +746,7 @@ class Abstract_Wallet(object):
             # save
             self.transactions[tx_hash] = tx
 
-    def remove_transaction(self, tx_hash, tx_height):
+    def remove_transaction(self, tx_hash):
         with self.transaction_lock:
             print_error("removing tx from history", tx_hash)
             #tx = self.transactions.pop(tx_hash)
@@ -770,13 +772,11 @@ class Abstract_Wallet(object):
 
 
     def receive_tx_callback(self, tx_hash, tx, tx_height):
-        self.add_transaction(tx_hash, tx, tx_height)
-        #self.network.pending_transactions_for_notifications.append(tx)
+        self.add_transaction(tx_hash, tx)
         self.add_unverified_tx(tx_hash, tx_height)
 
 
     def receive_history_callback(self, addr, hist):
-
         with self.lock:
             old_hist = self.history.get(addr, [])
             for tx_hash, height in old_hist:
@@ -784,7 +784,7 @@ class Abstract_Wallet(object):
                     # remove tx if it's not referenced in histories
                     self.tx_addr_hist[tx_hash].remove(addr)
                     if not self.tx_addr_hist[tx_hash]:
-                        self.remove_transaction(tx_hash, height)
+                        self.remove_transaction(tx_hash)
 
             self.history[addr] = hist
             self.storage.put('addr_history', self.history, True)
@@ -800,7 +800,7 @@ class Abstract_Wallet(object):
             tx = self.transactions.get(tx_hash)
             if tx is not None and self.txi.get(tx_hash, {}).get(addr) is None and self.txo.get(tx_hash, {}).get(addr) is None:
                 tx.deserialize()
-                self.add_transaction(tx_hash, tx, tx_height)
+                self.add_transaction(tx_hash, tx)
 
 
     def get_history(self, domain=None):
@@ -878,6 +878,7 @@ class Abstract_Wallet(object):
         # this method can be overloaded
         return tx.get_fee()
 
+    @profiler
     def estimated_fee(self, tx, fee_per_kb):
         estimated_size = len(tx.serialize(-1))/2
         fee = int(fee_per_kb * estimated_size / 1000.)
@@ -893,32 +894,45 @@ class Abstract_Wallet(object):
 
         fee_per_kb = self.fee_per_kb(config)
         amount = sum(map(lambda x:x[2], outputs))
-        total = fee = 0
+        total = 0
         inputs = []
         tx = Transaction.from_io(inputs, outputs)
-        # add old inputs first
+        fee = fixed_fee if fixed_fee is not None else 0
+        # add inputs, sorted by age
         for item in coins:
             v = item.get('value')
             total += v
             self.add_input_info(item)
             tx.add_input(item)
             # no need to estimate fee until we have reached desired amount
-            if total < amount:
+            if total < amount + fee:
                 continue
             fee = fixed_fee if fixed_fee is not None else self.estimated_fee(tx, fee_per_kb)
             if total >= amount + fee:
                 break
         else:
             raise NotEnoughFunds()
-        # remove unneeded inputs
+        # remove unneeded inputs.
+        removed = False
         for item in sorted(tx.inputs, key=itemgetter('value')):
             v = item.get('value')
             if total - v >= amount + fee:
                 tx.inputs.remove(item)
                 total -= v
-                fee = fixed_fee if fixed_fee is not None else self.estimated_fee(tx, fee_per_kb)
+                removed = True
+                continue
             else:
                 break
+        if removed:
+            fee = fixed_fee if fixed_fee is not None else self.estimated_fee(tx, fee_per_kb)
+            for item in sorted(tx.inputs, key=itemgetter('value')):
+                v = item.get('value')
+                if total - v >= amount + fee:
+                    tx.inputs.remove(item)
+                    total -= v
+                    fee = fixed_fee if fixed_fee is not None else self.estimated_fee(tx, fee_per_kb)
+                    continue
+                break
         print_error("using %d inputs"%len(tx.inputs))
 
         # change address
